package com.yifu.cloud.plus.v1.yifu.archives.vo;

import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Past;
import javax.validation.constraints.Pattern;
import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * 员工伤残excel 对应的实体
 *
 * @author fxz
 * @date 2022/3/21
 */
@Data
@ColumnWidth(30)
public class EmpDisabilityExcelVO implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * 员工编码
	 */
	@NotNull(message = "员工编码不能为空")
	@ExcelProperty(value ="员工编码")
	private String empCode;
	/**
	 * 员工姓名
	 */
	@NotNull(message = "员工姓名不能为空")
	@ExcelProperty(value ="员工姓名")
	private String empName;
	/**
	 * 伤残病名称
	 */
	@NotNull(message = "伤残病名称不能为空")
	@ExcelProperty(value ="伤残病名称")
	private String disabilityName;

	/**
	 * 伤残等级
	 */
	@NotNull(message = "伤残等级不能为空")
	@ExcelProperty(value ="伤残等级")
	private String disabilityLevel;

	/**
	 * 是否工伤标识0是/1否
	 */
	@NotNull(message = "是否工伤标识(0是/1否)不能为空")
	@ExcelProperty(value ="是否工伤")
	private String injuryIdentification;

	/**
	 * 伤残开始日期
	 */
	@NotNull(message = "伤残开始日期不能为空")
	@Past
	@ExcelProperty(value ="伤残开始日期")
	private LocalDateTime startTime;

	/**
	 * 伤残结束日期
	 */
	@NotNull(message = "伤残结束日期不能为空")
	@Past
	@ExcelProperty(value ="伤残结束日期")
	private LocalDateTime endTime;

	/**
	 * 职业病名称
	 */
	@ExcelProperty(value ="职业病名称")
	private String occupationalDisease;

	/**
	 * 备注
	 */
	@ExcelProperty(value ="备注")
	private String remark;

	/*JSR303 是一套JavaBean参数校验的标准，它定义了很多常用的校验注解，我们可以直接将这些注解加在我们JavaBean的属性上面，就可以在需要校验的时候进行校验了。注解如下：
	参考配置
	@Null 限制只能为null
	@NotNull 限制必须不为null
	@AssertFalse 限制必须为false
	@AssertTrue 限制必须为true
	@DecimalMax(value) 限制必须为一个不大于指定值的数字
	@DecimalMin(value) 限制必须为一个不小于指定值的数字
	@Digits(integer,fraction) 限制必须为一个小数，且整数部分的位数不能超过integer，小数部分的位数不能超过fraction
	@Future 限制必须是一个将来的日期
	@Max(value) 限制必须为一个不大于指定值的数字
	@Min(value) 限制必须为一个不小于指定值的数字
	@Past 限制必须是一个过去的日期
	@Pattern(value) 限制必须符合指定的正则表达式
	@Size(max,min) 限制字符长度必须在min到max之间
	@Past 验证注解的元素值（日期类型）比当前时间早
	@NotEmpty 验证注解的元素值不为null且不为空（字符串长度不为0、集合大小不为0）
	@NotBlank 验证注解的元素值不为空（不为null、去除首位空格后长度为0），不同于@NotEmpty，@NotBlank只应用于字符串且在比较时会去除字符串的空格
	@Email 验证注解的元素值是Email，也可以通过正则表达式和flag指定自定义的email格式*/

}
