/*
 *    Copyright (c) 2018-2025, lengleng All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the yifu4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: lengleng (wangiegie@gmail.com)
 */

package com.yifu.cloud.plus.v1.yifu.archives.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.pig4cloud.plugin.excel.annotation.RequestExcel;
import com.pig4cloud.plugin.excel.vo.ErrorMessage;
import com.yifu.cloud.plus.v1.yifu.archives.entity.TEmpFamily;
import com.yifu.cloud.plus.v1.yifu.archives.service.TEmpFamilyService;
import com.yifu.cloud.plus.v1.yifu.archives.vo.EmpFamilyExcelVO;
import com.yifu.cloud.plus.v1.yifu.common.core.constant.CommonConstants;
import com.yifu.cloud.plus.v1.yifu.common.core.util.Common;
import com.yifu.cloud.plus.v1.yifu.common.core.util.R;
import com.yifu.cloud.plus.v1.yifu.common.log.annotation.SysLog;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 员工家庭信息表
 *
 * @author fxj
 * @date 2022-06-18 09:39:46
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/tempfamily" )
@Tag(name = "员工家庭信息表管理")
public class TEmpFamilyController {

    private final TEmpFamilyService tEmpFamilyService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param tEmpFamily 员工家庭信息表
     * @return
     */
    @Operation(description = "分页查询")
    @GetMapping("/page" )
    public R<IPage<TEmpFamily>> getTEmpFamilyPage(Page page, TEmpFamily tEmpFamily) {
        return R.ok(tEmpFamilyService.page(page, Wrappers.query(tEmpFamily)));
    }
	/**
	 * 不分页查询
	 * @param tEmpFamily 员工家庭信息表
	 * @return
	 */
	@Operation(description = "不分页查询")
	@GetMapping("/noPage" )
	public R<List<TEmpFamily>> getTEmpFamilyNoPage(TEmpFamily tEmpFamily) {
		return R.ok(tEmpFamilyService.list( Wrappers.query(tEmpFamily)));
	}

    /**
     * 通过id查询员工家庭信息表
     * @param id id
     * @return R
     */
    @Operation(description = "通过id查询hasPermission('archives-family-detail')")
    @GetMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('archives-family-detail')" )
    public R<TEmpFamily> getById(@PathVariable("id" ) Long id) {
        return R.ok(tEmpFamilyService.getById(id));
    }

    /**
     * 新增员工家庭信息表  // TODO 新增变更档案学历最高学历信息
     * @param tEmpFamily 员工家庭信息表
     * @return R
     */
    @Operation(description = "新增员工家庭信息表hasPermission('archives-family-add')")
    @SysLog("新增员工家庭信息表" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('archives-family-add')" )
    public R<Boolean> save(@RequestBody TEmpFamily tEmpFamily) {
		String checkRes = tEmpFamilyService.checkRepeat(tEmpFamily.getEmpIdcard(),tEmpFamily.getFamilyName(),null);
		if (Common.isNotNull(checkRes)){
			return R.failed(checkRes);
		}
		tEmpFamily.setDeleteFlag(CommonConstants.ZERO_STRING);
        return R.ok(tEmpFamilyService.save(tEmpFamily));
    }

    /**
     * 修改员工家庭信息表
     * @param tEmpFamily 员工家庭信息表
     * @return R
     */
    @Operation(description = "修改员工家庭信息表hasPermission('archives-family-edit')")
    @SysLog("修改员工家庭信息表" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('archives-family-edit')" )
    public R<Boolean> updateById(@RequestBody TEmpFamily tEmpFamily) {
		String checkRes = tEmpFamilyService.checkRepeat(tEmpFamily.getEmpIdcard(),tEmpFamily.getFamilyName(),tEmpFamily.getId());
		if (Common.isNotNull(checkRes)){
			return R.failed(checkRes);
		}
        return R.ok(tEmpFamilyService.updateById(tEmpFamily));
    }

    /**
     * 通过id删除员工家庭信息表
     * @param id id
     * @return R
     */
    @Operation(description = "通过id删除员工家庭信息表hasPermission('archives-family-delete')")
    @SysLog("通过id删除员工家庭信息表" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('archives-family-delete')" )
    public R<Boolean> removeById(@PathVariable String id) {
        return R.ok(tEmpFamilyService.removeById(id));
    }


	/**
	 * 通过id删除员工家庭信息表
	 * @param ids
	 * @return R
	 */
	@Operation(description = "通过id删除员工家庭信息表hasPermission('archives-family-batch-delete')")
	@SysLog("通过id删除员工家庭信息表" )
	@DeleteMapping("/removeByIds" )
	@PreAuthorize("@pms.hasPermission('archives-family-batch-delete')" )
	public R<Boolean> removeByIds(String ids) {
		List<String> idList = null;
		if (Common.isNotNull(ids)){
			idList = Common.initStrToList(ids, CommonConstants.COMMA_STRING);
		}
		return R.ok(tEmpFamilyService.removeByIds(idList));
	}


	/**
	 * 批量新增员工家庭信息
	 *
	 * @param excelVOList
	 * @return
	 * @Author fxj
	 * @Date 2019-08-16
	 **/
	@Operation(description = "批量新增员工家庭信息 hasPermission('archives-family-batch-import')")
	@SysLog("批量新增员工家庭信息")
	@PostMapping("/importListAdd")
	@PreAuthorize("@pms.hasPermission('archives-family-batch-import')")
	public R<List<ErrorMessage>> importListAdd(@RequestExcel List<EmpFamilyExcelVO> excelVOList, BindingResult bindingResult) {
		return tEmpFamilyService.importEmpFamilys(excelVOList, bindingResult);
	}
}
