/*
 *    Copyright (c) 2018-2025, lengleng All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the yifu4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: lengleng (wangiegie@gmail.com)
 */

package com.yifu.cloud.plus.v1.yifu.archives.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yifu.cloud.plus.v1.yifu.archives.entity.TEmpProfessionalQualification;
import com.yifu.cloud.plus.v1.yifu.archives.service.TEmpProfessionalQualificationService;
import com.yifu.cloud.plus.v1.yifu.common.core.util.R;
import com.yifu.cloud.plus.v1.yifu.common.log.annotation.SysLog;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 员工职业资格信息表
 *
 * @author fxj
 * @date 2022-06-18 09:39:46
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/tempprofessionalqualification" )
@Tag(name = "员工职业资格信息表管理")
public class TEmpProfessionalQualificationController {

    private final TEmpProfessionalQualificationService tEmpProfessionalQualificationService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param tEmpProfessionalQualification 员工职业资格信息表
     * @return
     */
    @Operation(description = "分页查询")
    @GetMapping("/page" )
    public R<IPage<TEmpProfessionalQualification>> getTEmpProfessionalQualificationPage(Page page, TEmpProfessionalQualification tEmpProfessionalQualification) {
        return R.ok(tEmpProfessionalQualificationService.page(page, Wrappers.query(tEmpProfessionalQualification)));
    }
	/**
	 * 不分页查询
	 * @param tEmpProfessionalQualification 员工职业资格信息表
	 * @return
	 */
	@Operation(description = "不分页查询")
	@GetMapping("/noPage" )
	public R<List<TEmpProfessionalQualification>> getTEmpProfessionalQualificationNoPage(TEmpProfessionalQualification tEmpProfessionalQualification) {
		return R.ok(tEmpProfessionalQualificationService.list(Wrappers.query(tEmpProfessionalQualification)));
	}

    /**
     * 通过id查询员工职业资格信息表
     * @param id id
     * @return R
     */
    @Operation(description = "通过id查询hasPermission('archives_tempprofessionalqualification_get')")
    @GetMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('archives_tempprofessionalqualification_get')" )
    public R getById(@PathVariable("id" ) String id) {
        return R.ok(tEmpProfessionalQualificationService.getById(id));
    }

    /**
     * 新增员工职业资格信息表
     * @param tEmpProfessionalQualification 员工职业资格信息表
     * @return R
     */
    @Operation(description = "新增员工职业资格信息表hasPermission('archives_tempprofessionalqualification_add')")
    @SysLog("新增员工职业资格信息表" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('archives_tempprofessionalqualification_add')" )
    public R<Boolean> save(@RequestBody TEmpProfessionalQualification tEmpProfessionalQualification) {
        return tEmpProfessionalQualificationService.saveDiy(tEmpProfessionalQualification);
    }

    /**
     * 修改员工职业资格信息表
     * @param tEmpProfessionalQualification 员工职业资格信息表
     * @return R
     */
    @Operation(description = "修改员工职业资格信息表hasPermission('archives_tempprofessionalqualification_edit')")
    @SysLog("修改员工职业资格信息表" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('archives_tempprofessionalqualification_edit')" )
    public R updateById(@RequestBody TEmpProfessionalQualification tEmpProfessionalQualification) {
        return R.ok(tEmpProfessionalQualificationService.updateById(tEmpProfessionalQualification));
    }

    /**
     * 通过id删除员工职业资格信息表
     * @param id id
     * @return R
     */
    @Operation(description = "通过id删除员工职业资格信息表hasPermission('archives_tempprofessionalqualification_del')")
    @SysLog("通过id删除员工职业资格信息表" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('archives_tempprofessionalqualification_del')" )
    public R removeById(@PathVariable String id) {
        return R.ok(tEmpProfessionalQualificationService.removeById(id));
    }

}
